const St      = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Tweener = imports.ui.tweener;
const Main    = imports.ui.main;

const LEFT  = true;
const RIGHT = false;

function ripples(left) {
    let ripples = [];
    for (let i = 0; i < 3; i++) {
	    ripples[i] = new St.BoxLayout({
	        style_class : (left ? 'norm-ripple' : 'inv-ripple'),
	        visible     : false,
	        opacity     : 0
	    });
    }
    return ripples;
}

function corner(left) {
    return new St.Bin({
	    style_class : 'dot',
	    track_hover : true,
	    can_focus   : true,
	    reactive    : true,
	    x_fill      : true,
	    y_fill      : true,
	    opacity     : 0
    });
}

function animate(left) {
    let monitor = Main.layoutManager.primaryMonitor;

    function anim(corner, gravity, ripple,
		          delay, time, startScale, startOpacity, finalScale) {
	    ripple._opacity = startOpacity;
	    ripple.set_anchor_point_from_gravity(gravity);
	    
	    ripple.visible = true;
	    ripple.opacity = 255 * Math.sqrt(startOpacity);
	    ripple.scale_x = ripple.scale_y = startScale;
	    
	    let [x, y] = corner.get_transformed_position();
	    ripple.x = x;
	    ripple.y = y;
	    
	    Tweener.addTween(ripple, { 
	        _opacity: 0,
	        scale_x: finalScale,
	        scale_y: finalScale,
	        delay: delay,
	        time: time,
	        transition: 'linear',
	        onUpdate: function() {
		        ripple.opacity = 255 * Math.sqrt(ripple._opacity);
	        },
	        onComplete: function() {
		        ripple.visible = false;
	        }
	    });
    };

    return function(corner, ripples) {
	    let delays         = [0.00, 0.05, 0.35];
	    let times          = [0.83, 1.00, 1.00];
	    let startScales    = [0.25, 0.00, 0.00];
	    let startOpacities = [1.00, 0.70, 0.30];
	    let finalScales    = [1.50, 1.25, 1.00];

	    let gravity;
	    let ltr = corner.get_text_direction() == Clutter.TextDirection.LTR;

	    if (left && ltr || !left && !ltr) {
	        corner.set_position(0, 0);
	        gravity = Clutter.Gravity.NORTH_WEST
	    } else {
	        corner.set_position(monitor.width - 1, 0);
	        gravity = Clutter.Gravity.NORTH_EAST;
	    }

	    for (let i = 0; i < ripples.length; ++i) {
	        let j = i % 3;
	        anim(corner, gravity, ripples[i],
		         delays[j], times[j], startScales[j], startOpacities[j], finalScales[j]);
	    }
    };
}
